package com.gnarly.engine.display;

import static org.lwjgl.glfw.GLFW.GLFW_FALSE;
import static org.lwjgl.glfw.GLFW.GLFW_PRESS;
import static org.lwjgl.glfw.GLFW.GLFW_RESIZABLE;
import static org.lwjgl.glfw.GLFW.GLFW_TRUE;
import static org.lwjgl.glfw.GLFW.GLFW_VISIBLE;
import static org.lwjgl.glfw.GLFW.glfwCreateWindow;
import static org.lwjgl.glfw.GLFW.glfwDestroyWindow;
import static org.lwjgl.glfw.GLFW.glfwGetKey;
import static org.lwjgl.glfw.GLFW.glfwGetMouseButton;
import static org.lwjgl.glfw.GLFW.glfwGetPrimaryMonitor;
import static org.lwjgl.glfw.GLFW.glfwGetVideoMode;
import static org.lwjgl.glfw.GLFW.glfwInit;
import static org.lwjgl.glfw.GLFW.glfwMakeContextCurrent;
import static org.lwjgl.glfw.GLFW.glfwPollEvents;
import static org.lwjgl.glfw.GLFW.glfwSetCursorPosCallback;
import static org.lwjgl.glfw.GLFW.glfwSetErrorCallback;
import static org.lwjgl.glfw.GLFW.glfwSetWindowPos;
import static org.lwjgl.glfw.GLFW.glfwSetWindowShouldClose;
import static org.lwjgl.glfw.GLFW.glfwShowWindow;
import static org.lwjgl.glfw.GLFW.glfwSwapBuffers;
import static org.lwjgl.glfw.GLFW.glfwSwapInterval;
import static org.lwjgl.glfw.GLFW.glfwTerminate;
import static org.lwjgl.glfw.GLFW.glfwWindowHint;
import static org.lwjgl.glfw.GLFW.glfwWindowShouldClose;
import static org.lwjgl.opengl.GL11.GL_BLEND;
import static org.lwjgl.opengl.GL11.GL_COLOR_BUFFER_BIT;
import static org.lwjgl.opengl.GL11.GL_DEPTH_BUFFER_BIT;
import static org.lwjgl.opengl.GL11.GL_ONE_MINUS_SRC_ALPHA;
import static org.lwjgl.opengl.GL11.GL_SRC_ALPHA;
import static org.lwjgl.opengl.GL11.GL_TEXTURE_2D;
import static org.lwjgl.opengl.GL11.GL_VERSION;
import static org.lwjgl.opengl.GL11.glBlendFunc;
import static org.lwjgl.opengl.GL11.glClear;
import static org.lwjgl.opengl.GL11.glClearColor;
import static org.lwjgl.opengl.GL11.glEnable;
import static org.lwjgl.opengl.GL11.glGetString;
import static org.lwjgl.system.MemoryUtil.NULL;

import java.awt.Dimension;
import java.awt.Toolkit;

import org.joml.Vector3f;
import org.lwjgl.glfw.GLFWErrorCallback;
import org.lwjgl.glfw.GLFWVidMode;
import org.lwjgl.opengl.GL;

public class Window {

	private long window;
	
	private int width, height, mx, my;
	private boolean vSync;
	
	public Window(Camera camera, String name, float scale, boolean vSync, boolean resizable) {
		width = camera.getWidth();
		height = camera.getHeight();
		this.vSync = vSync;
		init(name, scale, resizable, false);
	}
	
	public Window(float percentWidth, float percentHeight, String name, boolean vSync, boolean resizable) {
		this.vSync = vSync;
		Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
		width = (int) (percentWidth / 100.0f * screen.getWidth());
		height = (int) (percentHeight / 100.0f * screen.getHeight());
		init(name, 1, resizable, false);
	}
	
	public Window(boolean vSync) {
		this.vSync = vSync;
		init("null", 1, false, true);
	}
	
	private void init(String name, float scale, boolean resizable, boolean fullscreen) {
		glfwSetErrorCallback(GLFWErrorCallback.createPrint(System.err));
		
		if(!glfwInit())
			throw new IllegalStateException("Could not initalize GLFW!");

		if(!fullscreen)
			glfwWindowHint(GLFW_RESIZABLE, resizable ? GLFW_TRUE : GLFW_FALSE);
		glfwWindowHint(GLFW_VISIBLE, GLFW_FALSE);
		
		GLFWVidMode vidMode = glfwGetVideoMode(glfwGetPrimaryMonitor());
		
		if(fullscreen || vidMode.width() < width || vidMode.height() < height) {
			width = vidMode.width();
			height = vidMode.height();
			window = glfwCreateWindow(width, height, name, glfwGetPrimaryMonitor(), NULL);
		}
		else {
			window = glfwCreateWindow((int)(width * scale), (int)(height * scale), name, NULL, NULL);
		
			glfwSetWindowPos(window, (int)((vidMode.width() - (width * scale)) / 2), (int)((vidMode.height() - (height * scale)) / 2));
		}

		glfwSetCursorPosCallback(window, (long window, double xpos, double ypos) -> {
			mx = (int)xpos;
			my = (int)ypos;
		});
		
		glfwMakeContextCurrent(window);
		GL.createCapabilities();
		
		if(vSync)
			glfwSwapInterval(1);
		
		glfwShowWindow(window);
		
		glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
		
		glEnable(GL_TEXTURE_2D);
		
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glEnable(GL_BLEND);
		
		System.out.println("OpenGL Version: " + glGetString(GL_VERSION));
	}
	
	public void update() {
		glfwPollEvents();
	}
	
	public void clear() {
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	}
	
	public void swap() {
		glfwSwapBuffers(window);
	}
	
	public Vector3f getMouseCoords() {
		return new Vector3f(mx, my, 0);
	}
	
	public int isMousePressed() {
		int button = -1;
		for (int i = 0; i < 8 && button == -1; i++)
			if(glfwGetMouseButton(window, i) == GLFW_PRESS)
				button = i;
		return button;
	}
	
	public boolean isKeyPressed(int keyCode) {
		return glfwGetKey(window, keyCode) == GLFW_PRESS;
	}
	
	public void close() {
		glfwSetWindowShouldClose(window, true);
	}
	
	public boolean shouldClose() {
		boolean close = glfwWindowShouldClose(window);
		if(close) {
			glfwDestroyWindow(window);
			glfwTerminate();
		}
		return close;
	}
	
	public static void cleanup() {
		
	}
	
	public long getWindow() {
		return window;
	}

	public int getWidth() {
		return width;
	}

	public int getHeight() {
		return height;
	}
}
