package com.gnarly.engine.utils;

import java.io.File;
import java.util.ArrayList;

import com.gnarly.engine.components.Shader;
import com.gnarly.engine.components.Texture;

public class Library {

	private static Texture[] textures;
	private static Shader[] shaders;
	private static File[] hitboxes;
	
	private Library() {}
	
	public static void init(String texStart, String shaderStart, String hitboxStart) {
		loadTextures(texStart);
		loadShaders(shaderStart);
		loadHitboxes(hitboxStart);
	}
	
	private static void loadTextures(String texStart) {
		String[] paths = loadStructure(texStart);
		textures = new Texture[paths.length];
		for (int i = 0; i < paths.length; i++)
			textures[i] = new Texture(paths[i]);
	}
	
	private static void loadShaders(String shaderStart) {
		String[] paths = loadStructure(shaderStart);
		shaders = new Shader[paths.length / 2];
		for (int i = 0; i < paths.length; i += 2)
			shaders[i / 2] = new Shader(paths[i + 1], paths[i]);
	}
	
	private static void loadHitboxes(String hitboxStart) {
		String[] paths = loadStructure(hitboxStart);
		hitboxes = new File[paths.length];
		for (int i = 0; i < paths.length; i++)
			hitboxes[i] = new File(paths[i]);
	}
	
	private static String[] loadStructure(String structureStart) {
		ArrayList<String> check = new ArrayList<>();
		ArrayList<String> files = new ArrayList<>();
		check.add(structureStart);
		while(check.size() > 0) {
			File file = new File(check.get(0));
			String path = file.getPath();
			String[] paths = file.list();
			if(paths != null) {
				for (int i = 0; i < paths.length; i++)
					check.add(path + "/" + paths[i]);
			}
			else
				files.add(path);
			check.remove(0);
		}
		String[] ret = new String[files.size()];
		for (int i = 0; i < ret.length; i++)
			ret[i] = files.get(i);
		return ret;
	}
	
	public static Texture getTexture(String name) {
		for (int i = 0; i < textures.length; i++)
			if(textures[i].getName().equals(name))
				return textures[i];
		throw new RuntimeException("Could not find texture: " + name + "!");
	}
	
	public static Shader getShader(String name) {
		for (int i = 0; i < shaders.length; i++)
			if(shaders[i].getName().equals(name))
				return shaders[i];
		throw new RuntimeException("Could not find shader: " + name + "!");
	}
	
	public static File getHitboxes(String name) {
		for (int i = 0; i < hitboxes.length; i++)
			if(hitboxes[i].getName().equals(name))
				return hitboxes[i];
		throw new RuntimeException("Could not find hitboxes: " + name + "!");
	}
}
