package com.gnarly.game;

import com.gnarly.engine.display.Camera;
import com.gnarly.engine.display.Window;
import com.gnarly.engine.utils.Library;
import com.gnarly.engine.utils.MemoryUtils;
import com.gnarly.game.panels.PlayPanel;

public class Main implements Runnable {

	private final int UPS = 120;
	private final int FPS = 120;
	
	private final int PIXEL = 5;
	
	private Window window;
	private Camera camera;
	private Thread gameLoop;
	private PlayPanel panel;
	
	public Main() {
		gameLoop = new Thread(this);
		gameLoop.start();
	}
	
	public void run() {
		init();
		float pastUTime = (float) System.nanoTime() / 1000000.0f;
		float curUTime = 0.0f;
		float mspu = 1000.0f / (float)UPS;
		float pastFTime = pastUTime;
		float curFTime = 0.0f;
		float mspf =  1000.0f / (float)FPS;
		float pastSec = pastUTime / 1000f;
		float curSec = 0.0f;
		int frames = 0;
		int updates = 0;
		while(!window.shouldClose()) {
			curUTime = (float) System.nanoTime() / 1000000.0f;
			curFTime = curUTime;
			if(curUTime - pastUTime > mspu) {
				update();
				pastUTime += mspu;
				++updates;
			}
			if(curFTime - pastFTime > mspf) {
				render();
				pastFTime += mspf;
				++frames;
			}
			curSec = (float) System.nanoTime() / 1000000000.0f;
			if(curSec - pastSec > 1.0f) {
				System.out.println(frames);
				frames = 0;
				updates = 0;
				pastSec += 1;
			}
		}
		MemoryUtils.destroy();
	}
	
	private void init() {
		camera = new Camera(1920 / PIXEL, 1080 / PIXEL);
		window = new Window(true);
		Library.init("res/img", "res/shaders", "res/hitboxes");
		panel = new PlayPanel(window, camera);
	}
	
	private void update() {
		window.update();
		panel.update();
	}
	
	private void render() {
		window.clear();
		panel.render();
		window.swap();
	} 
	
	public static void main(String[] args) {
		new Main();
	}
}
