package com.gnarly.game.objects;

import com.gnarly.engine.components.Shader;
import com.gnarly.engine.components.VAO;
import com.gnarly.engine.display.Camera;
import com.gnarly.engine.utils.Library;

public class Rectangle {

	private float x, y, width, height;

	private Camera camera;
	private Shader shader;
	private VAO vao;
	
	public Rectangle(Camera camera, float x, float y, float width, float height) {
		this.camera = camera;
		this.x = x;
		this.y = y;
		this.width = width;
		this.height = height;
		shader = Library.getShader("shapes");
		float[] vertices = {
			0.0f,  0.0f,   1.0f, //TOP LEFT
			0.0f,  height, 1.0f, //BOTTOM LEFT
			width, height, 1.0f, //BOTTOM RIGHT
			width, 0.0f,   1.0f  //TOP RIGHT
		};
		int[] indices = {
			0, 1, 3,
			1, 2, 3
		};
		float[] texCoords = {
			0, 0,
			0, 1,
			1, 1,
			1, 0
		};
		vao = new VAO(vertices, indices, texCoords);
	}
	
	public void render() {
		shader.setUniformMat4f("projection", camera.getProjection().translate(x, y, 0));
		shader.enable();
		vao.render();
		shader.disable();
	}
}
