package com.gnarly.game.objects;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;

import org.joml.Matrix4f;
import org.joml.Vector3f;

import com.gnarly.engine.components.Animation;
import com.gnarly.engine.components.Shader;
import com.gnarly.engine.components.VAO;
import com.gnarly.engine.display.Camera;
import com.gnarly.engine.utils.Library;
import com.gnarly.engine.utils.RHitbox;

public class Stage {

	private float width, height, floor;
	private Vector3f center;
	
	private VAO vao;
	private Shader shader;
	private Animation stage;
	private RHitbox[] hitboxes;
	private Camera camera;
	
	public Stage(Camera camera, String stageName) {
		this.camera = camera;
		stage = new Animation("mountain", "png", 1, 10, true);
		shader = Library.getShader("default");
		width = stage.getWidth();
		height = stage.getHeight();
		try {
			File hitboxFile = Library.getHitboxes(stageName + ".shb");
			ObjectInputStream input = new ObjectInputStream(new FileInputStream(hitboxFile));
			center = new Vector3f(input.readFloat(), input.readFloat(), 0);
			hitboxes = new RHitbox[input.readInt()];
			for (int i = 0; i < hitboxes.length; i++)
				hitboxes[i] = new RHitbox(input.readFloat(), input.readFloat(), input.readFloat(), input.readFloat());
			input.close();
		} catch(IOException e) {
			e.printStackTrace();
		}
		floor = hitboxes[0].getY();
		float[] vertices = {
			0.0f,  0.0f,   0.0f, //TOP LEFT
			0.0f,  height, 0.0f, //BOTTOM LEFT
			width, height, 0.0f, //BOTTOM RIGHT
			width, 0.0f,   0.0f  //TOP RIGHT
		};
		int[] indices = {
			0, 1, 3,
			1, 2, 3
		};
		float[] texCoords = {
			0, 0,
			0, 1,
			1, 1,
			1, 0
		};
		vao = new VAO(vertices, indices, texCoords);
	}
	
	public void update() {
		stage.update();
	}
	
	public void render() {
		shader.setUniformMat4f("projection", camera.getProjection());
		stage.bind();
		shader.enable();
		vao.render();
		shader.disable();
		stage.unbind();
	}
	
	public boolean checkPlayer(RHitbox character) {
		Vector3f translate = new Vector3f();
		for (int i = 0; i < hitboxes.length && translate.x == 0 && translate.y == 0; i++)
			translate = character.collisionAdjust(hitboxes[i]);
		if(translate.x != 0 || translate.y != 0) {
			character.translate(translate);
			checkPlayer(character);
			return true;
		}
		else
			return false;
	}
	
	public float getFloor() {
		return floor;
	}
	
	public Vector3f getCenter() {
		return center;
	}
}
